#include <stdio.h>
#include "libmegapixels.h"


float
clamp_float(float value, float min, float max)
{
	if (value > max)
		return max;

	if (value < min)
		return min;

	return value;
}

void
libmegapixels_aaa_software_statistics(unsigned int *frame, const int width, const int height,
	libmegapixels_aaa_stats *stats)
{
	unsigned int bright = 0;
	unsigned int too_bright = 0;
	unsigned int too_dark = 0;
	unsigned int total = 0;
	unsigned long long sum_r = 0, sum_g = 0, sum_b = 0;
	for (ssize_t p = 0; p < width * height; p++) {
		total++;
		unsigned int r = (frame[p] >> 0) & 0xff;
		unsigned int g = (frame[p] >> 8) & 0xff;
		unsigned int b = (frame[p] >> 16) & 0xff;
		unsigned int y = (r + g + b) / 3;

		if (y > 220) {
			too_bright++;
		}

		if (y > 180) {
			bright++;
		}
		if (y < 2) {
			too_dark++;
		}

		// Whitebalance on the midrange pixels only
		if (y > 75 && y < 200) {
			sum_r += r;
			sum_g += g;
			sum_b += b;
		}
	}
	unsigned int p_bright = (bright * 100) / total;
	unsigned int p_too_bright = (too_bright * 100) / total;
	unsigned int p_dark = (too_dark * 100) / total;
	stats->exposure = 0;
	if (p_bright < 1) {
		stats->exposure = 1;
	}
	if (p_too_bright > 8) {
		stats->exposure = -1;
	}
	stats->blacklevel = 0;
	if (p_dark < 1) {
		stats->blacklevel = -1;
	}
	if (p_dark > 3) {
		stats->blacklevel = 1;
	}

	float y = (float) (sum_r * 0.2126 + sum_g * 0.7175 + sum_b * 0.0722);
	float r = (float) sum_r / y;
	float g = (float) sum_g / y;
	float b = (float) sum_b / y;
	if (y < 1) {
		r = 0;
		g = 0;
		b = 0;
	}
	float m = (r + b) / 2.0f;
	stats->temp = clamp_float(b - r, 0, 2);
	stats->tint = clamp_float(g - m, 0, 2);
}
